Package: RaggedExperiment
Title: Representation of Sparse Experiments and Assays Across Samples
Version: 1.34.0
Authors@R: c(
    person("Martin", "Morgan", , "martin.morgan@roswellpark.org", "aut"),
    person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu",
        c("aut", "cre"), c(ORCID = "0000-0002-3242-0582")),
    person("Lydia", "King", , "L.King18@nuigalway.ie", "ctb")
    )
Description: This package provides a flexible representation of copy
    number, mutation, and other data that fit into the ragged array
    schema for genomic location data. The basic representation of such
    data provides a rectangular flat table interface to the user with
    range information in the rows and samples/specimen in the columns.
    The RaggedExperiment class derives from a GRangesList representation
    and provides a semblance of a rectangular dataset.
License: Artistic-2.0
biocViews: Infrastructure, DataRepresentation
BugReports: https://github.com/Bioconductor/RaggedExperiment/issues
URL: https://bioconductor.github.io/RaggedExperiment,
        https://bioconductor.org/packages/RaggedExperiment
VignetteBuilder: knitr
Depends: R (>= 4.5.0), GenomicRanges (>= 1.61.1)
Imports: BiocBaseUtils, BiocGenerics, Seqinfo, IRanges, Matrix,
        MatrixGenerics, methods, S4Vectors, stats, SummarizedExperiment
        (>= 1.39.1), utils
Suggests: BiocStyle, knitr, rmarkdown, testthat, GenomeInfoDb,
        MultiAssayExperiment
RoxygenNote: 7.3.2
Encoding: UTF-8
Date: 2024-10-17
git_url: https://git.bioconductor.org/packages/RaggedExperiment
git_branch: RELEASE_3_22
git_last_commit: 2cf1b41
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:54:50 UTC; biocbuild
Author: Martin Morgan [aut],
  Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  Lydia King [ctb]
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>
