# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,RaggedExperiment)
export(RaggedExperiment)
export(compactAssay)
export(compactSummarizedExperiment)
export(disjoinAssay)
export(disjoinSummarizedExperiment)
export(qreduceAssay)
export(qreduceSummarizedExperiment)
export(sparseAssay)
export(sparseSummarizedExperiment)
exportClasses(RaggedExperiment)
exportMethods("$")
exportMethods("[")
exportMethods("colData<-")
exportMethods("dimnames<-")
exportMethods("mcols<-")
exportMethods("rowData<-")
exportMethods("rowRanges<-")
exportMethods("seqinfo<-")
exportMethods(as.data.frame)
exportMethods(as.list)
exportMethods(assay)
exportMethods(assayNames)
exportMethods(assays)
exportMethods(coerce)
exportMethods(colData)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(length)
exportMethods(mcols)
exportMethods(overlapsAny)
exportMethods(rowData)
exportMethods(rowRanges)
exportMethods(seqinfo)
exportMethods(show)
exportMethods(subset)
exportMethods(subsetByOverlaps)
import(BiocGenerics)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
import(SummarizedExperiment)
import(methods)
importClassesFrom(GenomicRanges,GRangesList)
importClassesFrom(S4Vectors,Annotated)
importFrom(BiocGenerics,relist)
importFrom(GenomicRanges,GRanges)
importFrom(MatrixGenerics,rowRanges)
importFrom(S4Vectors,mcols)
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,seqinfo)
importFrom(SummarizedExperiment,"rowRanges<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,rowData)
importFrom(stats,setNames)
importFrom(utils,.DollarNames)
